<?php

namespace Jaise\BeejeesLavalite\Providers;

use Illuminate\Support\ServiceProvider;

class BeejeesLavaliteServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'beejees_lavalite');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'beejees_lavalite');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerBeejeesLavalite();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerBeejeesLavalite()
    {
        $this->app->bind('beejees_lavalite', function($app) {
            return new BeejeesLavalite($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('BeejeesLavalite', 'Lavalite\BeejeesLavalite\Facades\BeejeesLavalite');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('jaise.beejees_lavalite', function ($app) {
            return $this->app->make('Jaise\BeejeesLavalite\BeejeesLavalite');
        });

        // Bind Test to repository
        $this->app->bind(
            'Jaise\BeejeesLavalite\Interfaces\TestRepositoryInterface',
            \Jaise\BeejeesLavalite\Repositories\Eloquent\TestRepository::class
        );        // Bind Test to repository
        $this->app->bind(
            'Jaise\BeejeesLavalite\Interfaces\TestRepositoryInterface',
            \Jaise\BeejeesLavalite\Repositories\Eloquent\TestRepository::class
        );

        $this->app->register(\Jaise\BeejeesLavalite\Providers\AuthServiceProvider::class);
                $this->app->register(\Jaise\BeejeesLavalite\Providers\EventServiceProvider::class);
        
        $this->app->register(\Jaise\BeejeesLavalite\Providers\RouteServiceProvider::class);
                // $this->app->register(\Jaise\BeejeesLavalite\Providers\WorkflowServiceProvider::class);
            }

    /**
     * Merges user's and beejees_lavalite's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'jaise.beejees_lavalite'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeBeejeesLavalite::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['jaise.beejees_lavalite'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('jaise/beejees_lavalite.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/beejees_lavalite')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/beejees_lavalite')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
